/*
 * Segment 2 (KEx),  SWE 2.DE Statischer Import MIF
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */


package de.bsvrz.kex.mif.konfigdaten;

import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.kex.mif.kb.KonfigurationsObjekt;
import de.bsvrz.kex.mif.mif.MifTyp;

/**
 * Schnittstelle des Moduls KonfigurationDaten der SWE.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public interface KonfigurationsDatenInterface {

	/**
	 * Die erzeugten Konfigurationsdaten werden als Konfigurationsbereich
	 * gespeichert.
	 */
	void exportiereKB();

	/**
	 * Mit dieser Methode wird die MIF-Datenbasis eingelesen.
	 *
	 * @param pfadname
	 *            Pfad/Name der einzulesenden MIF-Datenbasis.
	 *
	 * @throws ImportException
	 *             bei Ausnahmen
	 */
	void importiereMIF(String pfadname) throws ImportException;

	/**
	 * Die Methode initialisiert das Modul.
	 *
	 * @param davConnection
	 *            {@link ClientDavConnection}
	 * @param dateiNameKonfigurationsBereich
	 *            Dateiname der Konfigurationsbereichsdatei
	 * @param konfigurationsVerantwortlicher
	 *            KonfigurationsVerantwortlicher
	 * @param konfigurationsBereichsPid
	 *            PID des Konfigurationsbereichs
	 * @param konfigurationsBereichsName
	 *            Name des Konfigurationsbereichs
	 * @param konfigurationsBereichsInfo
	 *            Info zum Konfigurationsbereich
	 */
	void initialisiere(ClientDavConnection davConnection, String dateiNameKonfigurationsBereich,
			String konfigurationsVerantwortlicher, String konfigurationsBereichsPid, String konfigurationsBereichsName,
			String konfigurationsBereichsInfo);

	/**
	 * Mit dieser Methode werden die MIF-Objekte in das interne Format
	 * konvertiert. Die Liste der konvertierten Objekte wird zur&uuml;ckgegeben.
	 *
	 * @param pidspaltenname
	 *            zu verwendender Spaltenname zur PID-Erzeugung
	 * @param miftypen
	 *            die Liste der zu konvertierenden MIF-Objekttypen
	 * @param zieltyp
	 *            Zielobjekttyp im Datenkatalog
	 * @return Liste von konvertierten Objekten vom Typ
	 *         {@link MifGeoObjektFactory}
	 */
	List<KonfigurationsObjekt> konvertiere(String pidspaltenname, List<MifTyp> miftypen, String zieltyp);
}
