/*
 * Segment 2 (KEx),  SWE 2.DE Statischer Import MIF
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.mif.vew;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Vector;

import javax.swing.AbstractCellEditor;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumn;

import de.bsvrz.kex.mif.kb.KonfigurationsObjekt;
import de.bsvrz.kex.mif.konfigdaten.ImportException;
import de.bsvrz.kex.mif.konfigdaten.KonfigurationsDaten;
import de.bsvrz.kex.mif.mif.MifTyp;

/**
 * Realisierung des Objekteditors.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class ObjektEditor extends JFrame {

	/**
	 *
	 */
	private static final long serialVersionUID = -6441319591327493333L;

	private static final int SPALTE_JN = 0;

	private static final int SPALTE_PID = 1;

	private static final int SPALTE_NAME = 2;

	private static final int SPALTE_MIFTYP = 3;

	private static final int SPALTE_DAVTYP = 4;

	String[] _spaltenNamen = { "Konvertieren", "PID", "Name", "Objekttyp MIF", "Objekttyp GeoReferenz" };

	Vector<String> _spaltenNamenV = new Vector<String>(Arrays.asList(_spaltenNamen));

	List<KonfigurationsObjekt> _konfigurationsobjekte;

	// private Object _last_add_dir;
	private GridBagLayout _gridbaglayout;

	private JButton _button_export;
	private JPanel panel_convert_buttons;
	private JTable table = null;
	KonfigurationsDaten _konfigurationsDaten = null;

	JFrame _frame;

	/**
	 * @param konfigurationsdaten
	 */
	public ObjektEditor(final KonfigurationsDaten konfigurationsdaten) {
		super("MIF-Objekteditor"); //$NON-NLS-1$

		_frame = this;

		_konfigurationsDaten = konfigurationsdaten;
		_konfigurationsobjekte = new ArrayList<KonfigurationsObjekt>(_konfigurationsDaten.getKonfigurationsObjekte());

		Collections.sort(_konfigurationsobjekte, new Comparator<KonfigurationsObjekt>() {
			@Override
			public int compare(final KonfigurationsObjekt o1, final KonfigurationsObjekt o2) {
				return o1.getPid().compareTo(o2.getPid());
			}
		});

		// float fontsize = 14f;
		final int fontsize_i = 14;
		// UIDefaults uiDefaults = UIManager.getDefaults();
		//
		// uiDefaults.put( "Label.font",
		// ((Font)uiDefaults.get("Label.font")).deriveFont(fontsize) );
		//
		// uiDefaults.put( "TextField.font",
		// ((Font)uiDefaults.get("TextField.font")).deriveFont(fontsize) );
		//
		// uiDefaults.put( "Button.font",
		// ((Font)uiDefaults.get("Button.font")).deriveFont(fontsize) );
		//
		// uiDefaults.put( "ComboBox.font",
		// ((Font)uiDefaults.get("ComboBox.font")).deriveFont(fontsize) );
		//

		this._gridbaglayout = new GridBagLayout();

		setLayout(this._gridbaglayout);
		// setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// setLocation((this._SCREEN_WIDTH - getSize().width) / 2,
		// (this._SCREEN_HEIGHT - getSize().height) / 2);

		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(final WindowEvent e) {
				// System.out.println("Abmelden1");
				super.windowClosing(e);
			}
		});

		_button_export = new JButton("Konfigurationsbereich erzeugen"); //$NON-NLS-1$
		// this._button_bast.setPreferredSize(new Dimension(75, 25));
		_button_export.setEnabled(true);

		_button_export.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(final ActionEvent e) {
				_frame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				enableComponent(_frame, false);
				SwingUtilities.invokeLater(new Runnable() {
					@Override
					public void run() {
						try {
							_konfigurationsDaten.exportiereKB();
							_frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
							enableComponent(_frame, true);
						} catch (final Exception ex) {
							JOptionPane.showMessageDialog(_frame, ex.getMessage(), "Fehler", JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
							return;
						}

					}
				});
			}
		});

		panel_convert_buttons = new JPanel(new FlowLayout(FlowLayout.CENTER));
		panel_convert_buttons.add(this._button_export);

		table = new JTable(new MyTableModel());
		// table.setCellEditor(new MyTableCellEditor());

		// table.setForeground(SystemColor.textHighlight);
		table.getTableHeader().setFont(new Font(null, Font.BOLD, 14));
		// table.getTableHeader().setMaximumSize(new Dimension(20,200));
		// table.getTableHeader().setActionMap(arg0);
		// table.setFont(new Font(null, Font.PLAIN, fontsize_i ));
		table.setRowHeight(fontsize_i + 5);
		// table.setIntercellSpacing(new Dimension(2,2));
		// _dest_obj.setBackground(Color.white);
		final JScrollPane scrollPane = new JScrollPane(table);
		// table.setPreferredScrollableViewportSize(new Dimension(500, 70));

		// Spalte Konvertieren
		TableColumn col = table.getColumnModel().getColumn(ObjektEditor.SPALTE_JN);
		// konvColumn.setHeaderValue("Konvertieren");
		final String[] konvOptions = { "Ja", "Nein" }; //$NON-NLS-1$ //$NON-NLS-2$
		final JComboBox comboBox_k = new JComboBox(konvOptions);
		comboBox_k.setEditable(false);
		comboBox_k.setSelectedIndex(0);
		comboBox_k.setBackground(Color.white);
		// comboBox_k.setFont(new Font(null, Font.PLAIN, 18 ));
		comboBox_k.setFont(comboBox_k.getFont().deriveFont(Font.PLAIN));
		col.setCellEditor(new DefaultCellEditor(comboBox_k));

		// Zieltyp
		col = table.getColumnModel().getColumn(ObjektEditor.SPALTE_DAVTYP);
		// konvColumn.setHeaderValue("Konvertieren");
		final JComboBox comboBox_kb = new JComboBox();
		comboBox_kb.setEditable(false);
		// comboBox_kb.setSelectedIndex(0);
		comboBox_kb.setBackground(Color.white);
		// col.setCellEditor(new DefaultCellEditor(comboBox_kb));
		col.setCellEditor(new DavTypCellEditor());

		ObjektEditor.addComponent(this, this._gridbaglayout, scrollPane, 0, 0, GridBagConstraints.RELATIVE, 1, 0.5, 2.5,
				GridBagConstraints.BOTH);
		ObjektEditor.addComponent(this, this._gridbaglayout, this.panel_convert_buttons, 0, 1,
				GridBagConstraints.RELATIVE, 1, 0.5, 0.0, GridBagConstraints.BOTH);

		// table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		setSize(new Dimension(950, 600));

		// for(int i=0; i<table.getColumnModel().getColumnCount(); i++) {
		table.getColumnModel().getColumn(0).setPreferredWidth(10);
		table.getColumnModel().getColumn(1).setPreferredWidth(100);
		table.getColumnModel().getColumn(2).setPreferredWidth(100);
		table.getColumnModel().getColumn(3).setPreferredWidth(10);
		table.getColumnModel().getColumn(4).setPreferredWidth(100);

		// table.sizeColumnsToFit(false);

		// pack();
		// table.getModel().addTableModelListener(this);

		// in die Mitte
		final int sw = Toolkit.getDefaultToolkit().getScreenSize().width;
		final int sh = Toolkit.getDefaultToolkit().getScreenSize().height;
		setLocation((sw - getSize().width) / 2, (sh - getSize().height) / 2);

		setVisible(true);
	}

	private static void addComponent(final Container cont, final GridBagLayout gbl, final Component c, final int x,
			final int y, final int width, final int height, final double weightx, final double weighty,
			final int fill) {
		final GridBagConstraints gbc = new GridBagConstraints();
		gbc.insets = new Insets(10, 10, 10, 10);
		gbc.fill = fill;
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = width;
		gbc.gridheight = height;
		gbc.weightx = weightx;
		gbc.weighty = weighty;
		gbl.setConstraints(c, gbc);
		cont.add(c);
	}

	@SuppressWarnings("serial")
	class MyTableModel extends AbstractTableModel {
		@Override
		public boolean isCellEditable(final int row, final int col) {
			return col != ObjektEditor.SPALTE_MIFTYP;
		}

		@Override
		public String getColumnName(final int col) {
			return _spaltenNamen[col].toString();
		}

		@Override
		public int getRowCount() {
			return _konfigurationsobjekte.size();
		}

		@Override
		public int getColumnCount() {
			return _spaltenNamen.length;
		}

		@Override
		public Object getValueAt(final int row, final int col) {
			final KonfigurationsObjekt ko = _konfigurationsobjekte.get(row);
			if (col == ObjektEditor.SPALTE_JN) {
				if (ko.sollImportieren()) {
					return "Ja"; //$NON-NLS-1$
				}
				return "Nein"; //$NON-NLS-1$
			} else if (col == ObjektEditor.SPALTE_PID) {
				return ko.getPid();
			} else if (col == ObjektEditor.SPALTE_NAME) {
				return ko.getName();
			} else if (col == ObjektEditor.SPALTE_MIFTYP) {
				try {
					return MifTyp.bestimmeTyp(ko.getGeometrie());
				} catch (final ImportException e) {
					return "unbekannt"; //$NON-NLS-1$
				}
			} else if (col == ObjektEditor.SPALTE_DAVTYP) {
				return ko.getDavobjektyp();
			} else {
				return "????"; //$NON-NLS-1$
			}
		}

		@Override
		public void setValueAt(final Object value, final int row, final int col) {
			final KonfigurationsObjekt ko = _konfigurationsobjekte.get(row);
			if (col == ObjektEditor.SPALTE_JN) {
				if (value.toString().equals("Ja")) { //$NON-NLS-1$
					if (ko.getDavobjektyp().equals(KonfigurationsDaten.DAVTYP_UNBESTIMMT)) {
						JOptionPane.showMessageDialog(_frame,
								"Bitte wählen Sie zunächst einen 'Objekttyp GeoReferenz' aus", "Information", //$NON-NLS-1$ //$NON-NLS-2$
								JOptionPane.INFORMATION_MESSAGE);
						return;
					}
					ko.setImportieren(true);
				} else {
					ko.setImportieren(false);
				}

			} else if (col == ObjektEditor.SPALTE_PID) {
				if (_konfigurationsDaten.checkPid(value.toString())) {
					_konfigurationsDaten.updatePID(ko.getPid(), value.toString());
					ko.setPid(value.toString());
				} else {
					JOptionPane.showMessageDialog(_frame, "Die PID ist nicht eindeutig", "Fehler", //$NON-NLS-1$ //$NON-NLS-2$
							JOptionPane.ERROR_MESSAGE);
				}
			} else if (col == ObjektEditor.SPALTE_NAME) {
				ko.setName(value.toString());
			} else if (col == ObjektEditor.SPALTE_DAVTYP) {
				ko.setZieltyp(value.toString());
			}
			fireTableCellUpdated(row, col);
		}
	}

	/**
	 * Cell-Editor f&uuml;r den DaV-Zieltyp
	 */
	public class DavTypCellEditor extends AbstractCellEditor implements TableCellEditor {
		/**
		 *
		 */
		private static final long serialVersionUID = 1L;
		private final JComboBox cbox = new JComboBox();

		@Override
		public final Component getTableCellEditorComponent(final JTable etable, final Object value,
				final boolean isSelected, final int rowIndex, final int colIndex) {
			final KonfigurationsObjekt ko = _konfigurationsobjekte.get(rowIndex);
			List<String> typen;
			try {
				typen = _konfigurationsDaten.bestimmeMoeglicheDavTypen(MifTyp.bestimmeTyp(ko.getGeometrie()));
			} catch (final ImportException e1) {
				typen = new ArrayList<String>();
				typen.add(KonfigurationsDaten.DAVTYP_UNBESTIMMT);
			}
			cbox.removeAllItems();
			for (final String t : typen) {
				cbox.addItem(t);
			}

			cbox.setEditable(false);
			cbox.setSelectedIndex(0);
			cbox.setBackground(Color.white);

			cbox.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(final ActionEvent e) {
					stopCellEditing();
				}
			});

			return cbox;
		}

		@Override
		public final Object getCellEditorValue() {
			return cbox.getSelectedItem();
		}
	}

	final void enableComponent(final Container cont, final boolean enable) {
		for (final Component c : cont.getComponents()) {
			c.setEnabled(enable);
			if (c instanceof Container) {
				enableComponent((Container) c, enable);
			}
		}
	}

}
